#include <algorithm>

#include "level.hpp"
#include "level_encoding.hpp"


level::level()
{ reset(); }

void level::reset()
{
    cell dot; dot.type=cell::dot; dot.wall_dir=0;

    std::fill(&cells[0][0],
        &cells[conf::playground_unit_count_x][conf::playground_unit_count_y],
        dot);

    robot_x=conf::playground_unit_count_x/2;
    robot_y=conf::playground_unit_count_y/2;
    robot_dir=direction::up;
    points=100;
    max_program_size=4;
    nr_white_buttons=0;
}

dyn_level::dyn_level(direction::type dir, int rx, int ry)
{ reset(dir, rx, ry); }

void dyn_level::reset(direction::type dir, int rx, int ry)
{
    robot_dir=dir;
    robot_x=rx;
    robot_y=ry;

    nr_white_pressed=0;
    points=0;

    std::fill(&cells[0][0],
        &cells[conf::playground_unit_count_x][conf::playground_unit_count_y],
        0);
}

bool read(level* lev, std::FILE* fp)
{
    char c;
    int x=0, y=0;

    lev->reset();
    while(fscanf(fp, " %c ", &c)==1){
        switch(c){
            case charcode::white: lev->cells[x][y].type=level::cell::white; lev->nr_white_buttons++; break;
            case charcode::gray: lev->cells[x][y].type=level::cell::gray; break;
            case charcode::robot_up: lev->robot_x=x; lev->robot_y=y; lev->robot_dir=direction::up; break;
            case charcode::robot_down: lev->robot_x=x; lev->robot_y=y; lev->robot_dir=direction::down; break;
            case charcode::robot_left: lev->robot_x=x; lev->robot_y=y; lev->robot_dir=direction::left; break;
            case charcode::robot_right: lev->robot_x=x; lev->robot_y=y; lev->robot_dir=direction::right; break;

            case charcode::wall_l: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left; break;
            case charcode::wall_r: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::right; break;
            case charcode::wall_u: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::up; break;
            case charcode::wall_d: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::down; break;


            case charcode::wall_lr: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::right; break;
            case charcode::wall_ud: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::up|level::cell::down; break;

            case charcode::wall_lu: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::up; break;
            case charcode::wall_ld: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::down; break;
            case charcode::wall_ru: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::right|level::cell::up; break;
            case charcode::wall_rd: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::right|level::cell::down; break;

            case charcode::wall_lru: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::right|level::cell::up; break;
            case charcode::wall_lrd: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::right|level::cell::down; break;
            case charcode::wall_lud: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::up|level::cell::down; break;
            case charcode::wall_rud: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::right|level::cell::up|level::cell::down; break;

            case charcode::wall_lrud: lev->cells[x][y].type=level::cell::wall; lev->cells[x][y].wall_dir=level::cell::left|level::cell::right|level::cell::up|level::cell::down; break;
            default: break;
        }
        x++;
        if(x==conf::playground_unit_count_x){
            y++;
            x=0;
            if(y==conf::playground_unit_count_y) break;
        }
    }
    if(y!=conf::playground_unit_count_y) return false;

    char linebuffer[200];
    bool found_points=false, found_max=false;
    while(! feof(fp) && (!found_points || !found_max)){
        fgets(linebuffer, sizeof(linebuffer), fp);
        int res;
        if(!found_points && sscanf(linebuffer, "points %d", &res)==1){
            lev->points=res; found_points=true;
        } else if(!found_max && sscanf(linebuffer, "maxsize %d", &res)==1){
            lev->max_program_size=res; found_max=true;
        }
    }

    //sprintf(linebuffer, "POINTS: %d, MAXSIZE: %d, WHITE: %d",
    //lev->points, lev->max_program_size, lev->nr_white_buttons);
    //MessageBox(NULL, linebuffer, "", 0);
    return found_max&&found_points;
}

void dyn_level::unpress_all()
{
    for(int x=0; x<conf::playground_unit_count_x; ++x)
    for(int y=0; y<conf::playground_unit_count_y; ++y)
        cells[x][y] &= ~(cell_info::pressed);
}
